% make_AIS_flux_gate

clear 
close all
clc

addpath(genpath('D:/work/Fellowship_Leeds/Data'));


%% Load initial grounding line

% Start with the MEaSUREs grounding line
disp('loading initial grounding line...');
[gl_x,gl_y] = antbounds_data('gl','xy'); 

% Get indices of main ice sheet (not islands), so that we can remove them or adjust the
% position of those pixels independently from the islands
basins = shaperead('Basins_IMBIE_Antarctica_v02.shp');
islands_x = basins(1).X;
islands_y = basins(1).Y;
[~,main] = inpoly2([gl_x(:),gl_y(:)],[islands_x(:),islands_y(:)]);
main = logical(1-main); % invert
clear islands_x islands_y basins

% Remove islands 
gl_x = gl_x(main);
gl_y = gl_y(main);

% Before adjusting the position of the gate, increase the density of grounding
% line points, so that the migration is a bit smoother
disp('increasing density of initial grounding line points...');
s = regionprops(isfinite(gl_x),gl_x,'PixelIdxList');
region_x = cell(numel(s),1);
region_y = cell(numel(s),1);
for i = 1:numel(s)
    tmp_x = gl_x(s(i).PixelIdxList);
    tmp_y = gl_y(s(i).PixelIdxList);
    total_length = arclength(tmp_x(:),tmp_y(:)); % total length of the transect in metres
    xy = interparc(0:(250/total_length):1,tmp_x(:),tmp_y(:),'linear');
    region_x{i} = xy(:,1);
    region_y{i} = xy(:,2);
end
gl_x = cell2nancat(region_x);
gl_y = cell2nancat(region_y);

%% Load velocity grid

disp('loading reference velocity grid');
load('../out/pub/reference_grids//reference_velocity_200m.mat','vx','vy');
load('../out/pub/reference_grids/reference_thickness_200m.mat','x','y');
[X,Y] = meshgrid(x,y);

%% Calculate flow direction

disp('calculating flow direction everywhere');
theta = zeros(size(vx)).*NaN; % pre-allocate array
isf = isfinite(vx); % get indices of finite values
[vx,vy] = vxvy2uv(X,Y,vx,vy); % convert vx and vy to u and v
tmp = cart2compass(vx(isf),vy(isf)); % calculate flow direction for finite pixels
theta(isf) = tmp; % insert into to 2D grid
[vx,vy] = uv2vxvy(X,Y,vx,vy); % convert u and v back to vx and vy
clear tmp isf

% interpolate theta to grounding line locations
gl_theta = interp2(X,Y,theta,gl_x,gl_y);

%% migrate grounding lines 

disp('migrating grounding lines');

% I want to migrate the grounding line by 3-years with of ice flow and a
% minimum of 1 km, so set the minimum speed to 1000/3 m/yr
min_sp = 1000/3;
sp = hypot(vx,vy);
slow = sp<min_sp;
sp(slow) = min_sp;

% Adjust vx and vy using flow direction and adjusted speed
[vx_adj,vy_adj] = compass2cart(theta(slow),sp(slow));
[vx_adj,vy_adj] = uv2vxvy(X(slow),Y(slow),vx_adj,vy_adj);
vx_orig = vx(slow);
vy_orig = vy(slow);
vx(slow) = vx_adj;
vy(slow) = vy_adj;
clear vx_adj vy_adj

% migrate grounding line of the main ice sheet to create gates
dt = 0.1;% years
t = dt:dt:3; % max years
mig_x = gl_x;
mig_y = gl_y;
for i = 1:length(t) % move gates in increments of 0.1 years, to avoid big jumps and overlaps
    disp(['progress: ' num2str(i) '/' num2str(length(t))]);
    mig_vx = interp2(X,Y,vx,mig_x,mig_y);
    mig_vy = interp2(X,Y,vy,mig_x,mig_y);
    mig_x = mig_x + mig_vx.*-dt; % migrate upstream incrementally
    mig_y = mig_y + mig_vy.*-dt; % migrate upstream incrementally
end

% Reset vx and vy
vx(slow) = vx_orig;
vy(slow) = vy_orig;
clear vx_orig vy_orig

% Migration can cause some annoying overlaps with floating ice, but we can
% remove these with an intersection mask
p1 = polyshape(gl_x,gl_y,'KeepCollinearPoints',true);
%   There are lots of intersections with the grounding line for the main
%   ice sheet excluding wilkins island, which results in lots of
%   nan-separated vectors in the incorrect order. So let's start by
%   splitting up wilkins island and the main ice sheet
s = regionprops(isfinite(mig_x),mig_x,'PixelIdxList');
% make polyshapes
p2 = polyshape(mig_x(s(1).PixelIdxList),mig_y(s(1).PixelIdxList),'KeepCollinearPoints',true);
p3 = polyshape(mig_x(s(2).PixelIdxList),mig_y(s(2).PixelIdxList),'KeepCollinearPoints',true);
in_12 = intersect(p1,p2); % intersection polyshape of main ice sheet and grounding line (excluding small islands)
in_13 = intersect(p1,p3); % intersection polyshape of wilkins island and grounding line (excluding small islands)
%   Just take the largest continuous segment for the main ice sheet
%   intersection
in_x12 = in_12.Vertices(:,1);
in_y12 = in_12.Vertices(:,2);
s = regionprops(isfinite(in_x12),in_x12,'PixelIdxList');
[~,iv] = max(cellfun(@length,struct2cell(s)));
in_x12 = in_x12(s(iv).PixelIdxList);
in_y12 = in_y12(s(iv).PixelIdxList);
%   and repeat for wilkins island
in_x13 = in_13.Vertices(:,1);
in_y13 = in_13.Vertices(:,2);
s = regionprops(isfinite(in_x13),in_x13,'PixelIdxList');
[~,iv] = max(cellfun(@length,struct2cell(s)));
in_x13 = in_x13(s(iv).PixelIdxList);
in_y13 = in_y13(s(iv).PixelIdxList);

% % Set gates - only need to do this if including islands
% tmp_x = gl_x;
% tmp_y = gl_y;
% tmp_x(main) = NaN;
% tmp_y(main) = NaN;
% un = union(polyshape(tmp_x,tmp_y),in);
% gate_x = un.Vertices(:,1);
% gate_y = un.Vertices(:,2);
% clear un in tmp_x tmp_y mig_x mig_y

gate_x = [in_x12; NaN; in_x13];
gate_y = [in_y12; NaN; in_y13];

%% Finalise gates

disp('Making gates regularly spaced...');
% Do this for uniformly 100 m spacing
% s = regionprops(isfinite(gate_x),gate_x,'PixelIdxList');
% region_x = cell(numel(s),1);
% region_y = cell(numel(s),1);
% for i = 1:numel(s)
%     tmp_x = gate_x(s(i).PixelIdxList);
%     tmp_y = gate_y(s(i).PixelIdxList);
%     total_length = arclength(tmp_x(:),tmp_y(:)); % total length of the transect in metres
%     xy = interparc(0:(100/total_length):1,tmp_x(:),tmp_y(:),'linear');
%     region_x{i} = xy(:,1);
%     region_y{i} = xy(:,2);
% end
% gate_x = cell2nancat(region_x);
% gate_y = cell2nancat(region_y);

% Re-extract speed over gate
gate_vx = interp2(X,Y,vx,gate_x,gate_y);
gate_vy = interp2(X,Y,vy,gate_x,gate_y);
gate_sp = hypot(gate_vx,gate_vy);

% Create a mask to identify fast and slow gate pixels
mask = gate_sp>100;
mask = bwareaopen(mask, 2); % remove isolated fast pixels
mask = ~mask;
mask = bwareaopen(mask, 2); % remove isolated slow pixels
mask = ~mask;

% Set gate spacing to at 100 m over fast (100 m/yr) flowing ice and 200 m
% elsewhere
s = regionprops(isfinite(gate_x),'PixelIdxList');
region_x = cell(numel(s),1);
region_y = cell(numel(s),1);
for i = 1:numel(s)
    disp(['Progress: ' num2str(i) '/' num2str(numel(s))]);
    tmp_x = gate_x(s(i).PixelIdxList);
    tmp_y = gate_y(s(i).PixelIdxList);
    tmp_mask = mask(s(i).PixelIdxList);
    if sum(tmp_mask)==0 % all slow
        total_length = arclength(tmp_x(:),tmp_y(:)); % total length of the transect in metres
        xy = interparc(0:(200/total_length):1,tmp_x(:),tmp_y(:),'linear');
        new_x = xy(:,1);
        new_y = xy(:,2);
    elseif sum(tmp_mask)==length(tmp_mask) % all fast
        total_length = arclength(tmp_x(:),tmp_y(:)); % total length of the transect in metres
        xy = interparc(0:(100/total_length):1,tmp_x(:),tmp_y(:),'linear');
        new_x = xy(:,1);
        new_y = xy(:,2);
    else % mixture of fast and slow
        %   - check for isolated fast or slow bits and the end of the mask sample
        % that could have been nan-adjacent
        tmp_mask = bwareaopen(tmp_mask, 2); 
        tmp_mask = ~tmp_mask;
        tmp_mask = bwareaopen(tmp_mask, 2); 
        tmp_mask = ~tmp_mask;
        %   - get fast and slow subregions and sort
        fast = regionprops(tmp_mask==1,'PixelIdxList');
        slow = regionprops(tmp_mask==0,'PixelIdxList');
        spacing = [ones(numel(fast),1).*100;ones(numel(slow),1).*200]; % spacing vector
        com = [fast;slow]; % merge structures
        [ind1,iv] = sort(cellfun(@(v)v(1),struct2cell(com))); % first index in each region, sorted
        spacing = spacing(iv);
        com = com(iv);
        %   - set regular spacing
        new_x = cell(numel(com),1);
        new_y = cell(numel(com),1);
        for j = 1:numel(com)
            total_length = arclength(tmp_x(com(j).PixelIdxList),tmp_y(com(j).PixelIdxList)); % total length of the transect in metres
            xy = interparc(0:(spacing(j)/total_length):1,tmp_x(com(j).PixelIdxList),tmp_y(com(j).PixelIdxList),'linear');
            new_x{j} = xy(:,1);
            new_y{j} = xy(:,2);
        end
        new_x = vertcat(new_x{:});
        new_y = vertcat(new_y{:});
    end
    region_x{i} = new_x;
    region_y{i} = new_y;
end
gate_x = cell2nancat(region_x);
gate_y = cell2nancat(region_y);

%% Save

save('../out/int/gates/initial_flux_gate.mat','gate_x','gate_y');
% S.Geometry = 'line';
% S.X = gate_x;
% S.Y = gate_y;
% shapewrite(S,'flux_gates.shp');


