function calc_discharge_timeseries_ns(migration_distance)

if ispc
    addpath(genpath('D:/work/Fellowship_Leeds/Data'));
else
    addpath(genpath('/resstore/b0133/Data/MATLAB_tools'));
end

%% Set physical parameters

% Ice density
rhoi = 917; % kg/m^3

%% Load flux gate data

% Gate locations
g = load('../out/pub/gates/flux_gates_migrated.mat');
migration_distances = 0:200:3000; % metres
migration_pixels = find(migration_distances == migration_distance); % metres
gate_x = g.gate_x_migrate{migration_pixels};
gate_y = g.gate_y_migrate{migration_pixels};
[gate_lat,gate_lon] = ps2ll(gate_x,gate_y);
gate_nan = isnan(gate_x); % indices of gate nans
clear g

% Reference data (various versions of bed topograpghy, one surface, gate angle)
ref = load(['../out/int/gate_data/reference_gate_data_migrate_' num2str(migration_distance) 'm.mat'],...
    'gate_theta','gate_H_raw','gate_H_adj','gate_H_BM','gate_H_B2',...
    'gate_H_raw_err','gate_H_BM_err','gate_H_B2_err');

% ice velocity timeseries
V = load(['../out/int/gate_data/gate_vel_filled_migrate_' num2str(migration_distance) 'm_ns.mat'],...
    'vx_filled','vy_filled','vxE_filled','vyE_filled','time_vel','filled','source_vel');

% gate-to-grounding line thickness change due to SMB
load(['../out/int/gate_data/downstream_smb_correction_' num2str(migration_distance) 'm.mat']); % gate_smb

% dHdt at gate
dH0 = load(['../out/int/gate_data/gate_quadratic_dH_ts_' num2str(migration_distance) 'm.mat'],'gate_quad','gate_lin','gate_icept','gate_quad_err','gate_lin_err','gate_icept_err','gate_start_time');

% firn air content
FAC_FDMraw = load(['../out/int/gate_data/FAC_migrate_' num2str(migration_distance) 'm.mat']); % FAC from IMAU FDM
FAC_GSFCraw = load(['../out/int/gate_data/FAC_GSFC_migrate_' num2str(migration_distance) 'm.mat']); % FAC from GSFC with MERRA-2
FAC_GEMBraw = load(['../out/int/gate_data/FAC_GEMB_migrate_' num2str(migration_distance) 'm.mat']); % FAC from GEMB

% Calculate discharge at each velocity epoch, so set time here
time = V.time_vel;


%% Calculate cross-gate velocity
% The aim is to provide an estimate of the total discharge across the
% grounding line, so always use the 'filled' velocity. Later, we'll
% calculate how much of the discharge has been observed vs how much is
% 'filled'

gate_theta = repmat(ref.gate_theta,[1,length(time)]); % angle of flux gate; gate never changes, so keep constant through time and assume no error in this angle
V.ctv = abs(V.vx_filled.*sin(gate_theta) - V.vy_filled.*cos(gate_theta));
V.vxE_filled(V.vxE_filled>5e3) = abs(V.vx_filled(V.vxE_filled>5e3)).*0.1;
V.vyE_filled(V.vyE_filled>5e3) = abs(V.vy_filled(V.vyE_filled>5e3)).*0.1;
V.vx_min = V.vx_filled-V.vxE_filled;
V.vx_max = V.vx_filled+V.vxE_filled;
V.vy_min = V.vy_filled-V.vyE_filled;
V.vy_max = V.vy_filled+V.vyE_filled;
V.ctv_vxmin = abs(V.vx_min.*sin(gate_theta) - V.vy_filled.*cos(gate_theta));
V.ctv_vxmax = abs(V.vx_max.*sin(gate_theta) - V.vy_filled.*cos(gate_theta));
V.ctv_vymin = abs(V.vx_filled.*sin(gate_theta) - V.vy_min.*cos(gate_theta));
V.ctv_vymax = abs(V.vx_filled.*sin(gate_theta) - V.vy_max.*cos(gate_theta));
V.ctvE = sqrt( ...
      (V.ctv-V.ctv_vxmin).^2 ...
    + (V.ctv-V.ctv_vxmax).^2 ...
    + (V.ctv-V.ctv_vymin).^2 ...
    + (V.ctv-V.ctv_vymax).^2);
V.ctvE(V.ctvE>5e3) = V.ctv(V.ctvE>5e3).*0.1; % check for unrealistic errors
V.ctv_min = V.ctv - V.ctvE;
V.ctv_max = V.ctv + V.ctvE;
%V.ctv_min(V.ctv_min<0) = 0;
clear V.vx_min V.vx_max V.vy_min V.vy_max V.ctv_vxmin V.ctv_vxmax V.ctv_vymin V.ctv_vymax gate_theta

%% Evaluate elevation change fit at velocity epochs

% convert velocity time to required format to evaluate fit
dH0.t = (time-datenum(1990,1,15))/365; % 1:1/12:32; % years from 1991
dH0.t_sampling_original = 140/365;
dH0.t_sampling_hires = 1/12;

% Calculate dH timeseries: PIG test point
dH = (dH0.gate_quad.*dH0.t.^2) + (dH0.gate_lin.*dH0.t) + dH0.gate_icept;
dH_upper = ((dH0.gate_quad+dH0.gate_quad_err).*dH0.t.^2) + ((dH0.gate_lin+dH0.gate_lin_err).*dH0.t) + (dH0.gate_icept+dH0.gate_icept_err);
dH_lower = ((dH0.gate_quad-dH0.gate_quad_err).*dH0.t.^2) + ((dH0.gate_lin-dH0.gate_lin_err).*dH0.t) + (dH0.gate_icept-dH0.gate_icept_err);
dH_err = (dH0.t_sampling_hires  / dH0.t_sampling_original) .* ((abs(dH_upper - dH) + abs(dH - dH_lower)) ./ 2);

% Set reference time to May 9th 2015 (time of reference thickness)
[~,idx] = findNearest(time, datenum(2015,5,9));
dH = dH - dH(:,idx);

% Keep static (reference) thickness south of 81.5
dH(dH0.gate_start_time>=2010) = 0;
dH_err(dH0.gate_start_time>=2010) = 0;

dH(gate_nan,:) = NaN;
dH_err(gate_nan,:) = NaN;

clear dH0 

%% Interpolate firn corrections to velocity epochs
% pre-allocate
FAC_FDM = zeros(size(V.ctv)).*NaN;
FAC_GSFC = zeros(size(V.ctv)).*NaN;
FAC_GEMB = zeros(size(V.ctv)).*NaN;
% loop through gate pixels
for j = 1:length(gate_x)
    if isfinite(gate_x(j))
        
        % load FAC and interpolate to velocity/discharge timeseries
        FAC_ts = FAC_FDMraw.fac_monthly(j,:);
        FAC_ts(FAC_ts==0)=NaN;
        FAC_ts = interp1gap(FAC_FDMraw.fac_time,FAC_ts,time);
        % As the velocity time-series extends beyond the FAC timeseries,
        % the more recent FAC time points will be NaN. Fill those with the
        % monthly FAC climatology.
        isn = isnan(FAC_ts);
        if sum(isn)>0
            [~,M,~] = datevec(time(isn));
            FAC_ts(find(isn==1)) = FAC_FDMraw.fac_monthly_clim(j,M);
        end
        FAC_FDM(j,:) = FAC_ts;

        % Repeat for GSFC firn model
        FAC_ts = FAC_GSFCraw.fac_monthly(j,:);
        FAC_ts(FAC_ts==0)=NaN;
        FAC_ts = interp1gap(FAC_GSFCraw.fac_time,FAC_ts,time);
        isn = isnan(FAC_ts);
        if sum(isn)>0
            [~,M,~] = datevec(time(isn));
            FAC_ts(find(isn==1)) = FAC_GSFCraw.fac_monthly_clim(j,M);
        end
        FAC_GSFC(j,:) = FAC_ts;
        
        % Repeat for GEMB firn model
        FAC_ts = FAC_GEMBraw.fac_monthly(j,:);
        FAC_ts(FAC_ts==0)=NaN;
        FAC_ts = interp1gap(FAC_GEMBraw.fac_time,FAC_ts,time);
        isn = isnan(FAC_ts);
        if sum(isn)>0
            [~,M,~] = datevec(time(isn));
            FAC_ts(find(isn==1)) = FAC_GEMBraw.fac_monthly_clim(j,M);
        end
        FAC_GEMB(j,:) = FAC_ts;
    end
end
FDM_clim = FAC_FDMraw.fac_clim;
clear FAC_ts FAC_FDMraw FAC_GSFCraw FAC_GEMBraw M isn j

%% Calculate time varying ice thickness

% Loop through thickness datasets
H_datasets = {'FrankenBed','FrankenBedAdj','BedMachine','BedMap2'};
for i = 1:numel(H_datasets)
    
    % Load thickness dataset
    if strcmp(H_datasets{i},'FrankenBed')
        H = double(repmat(ref.gate_H_raw(:),[1,length(time)]));
        He = double(repmat(ref.gate_H_raw_err(:),[1,length(time)]));
    elseif strcmp(H_datasets{i},'FrankenBedAdj')
        H = double(repmat(ref.gate_H_raw(:)+H_change,[1,length(time)])); % note, we're modifying the 'raw' FrankenBed thickness here based on the differnece between the FrankenBed discharge and the discharge required to match observed dHdt from 1996-2021
        He = double(repmat(ref.gate_H_raw_err(:),[1,length(time)]));
    elseif strcmp(H_datasets{i},'BedMachine')
        H  = double(repmat(ref.gate_H_BM(:), [1,length(time)]));
        He  = double(repmat(ref.gate_H_BM_err(:), [1,length(time)]));
    elseif strcmp(H_datasets{i},'BedMap2')
        H  = double(repmat(ref.gate_H_B2(:), [1,length(time)]));
        He  = double(repmat(ref.gate_H_B2_err(:), [1,length(time)]));
    else
        disp('Unrecognised thickness dataset, exiting');
        quit
    end
    H0 = H;
    
    % Check for missing thickness error
    He(isnan(He) & isfinite(H)) = H(isnan(He) & isfinite(H)).*0.3; % assume very large (30%) error in these places

    % Apply thickness change
    dH = double(dH);
    H_static = H; % 'static'; no corrections applied
    H_insitu = H + dH; % 'in situ'; apply everything except the surface mass change between the gate and grounding line
    H_NDH = H; % 'No dH'; apply everything except time-varying dH
    H_NF = H + dH; % 'no firn'; apply everything except a firn correction
    H = H + dH; % apply everything

    % Add dH error to relevant thickness datasets, and initialise others.
    % Also assume 20% error in the firn air content correction
    dH_err      = double(dH_err);
    FAC_FDM     = double(FAC_FDM);
    FAC_GSFC    = double(FAC_GSFC);
    FAC_GEMB    = double(FAC_GEMB);
    He_static   = He;
    He_insitu   = sqrt(He.^2 + dH_err.^2 + FAC_FDM.*0.2);
    He_NDH      = sqrt(He.^2 + FAC_FDM.*0.2);
    He_NF       = sqrt(He.^2 + dH_err.^2);
    He_GSFC     = sqrt(He.^2 + dH_err.^2 + FAC_GSFC.*0.2); % initialise error for discharge using GSFC firn model; include dH error
    He_GEMB     = sqrt(He.^2 + dH_err.^2 + FAC_GEMB.*0.2); % initialise error for discharge using GEMB firn model; include dH error
    He          = sqrt(He.^2 + dH_err.^2 + FAC_FDM.*0.2);

    % Apply firn correction
    %   - the baseline thickness estimates above already have the
    %   climatological firn content removed, so we need to add that back on
    %   then apply the time-varying firn corrections
    FDM_clim = double(FDM_clim);
    H        = H        + repmat(FDM_clim,[1,length(time)]) - FAC_FDM;
    H_insitu = H_insitu + repmat(FDM_clim,[1,length(time)]) - FAC_FDM;
    H_NDH    = H_NDH    + repmat(FDM_clim,[1,length(time)]) - FAC_FDM;
    H_GSFC   = H        + repmat(FDM_clim,[1,length(time)]) - FAC_GSFC;
    H_GEMB   = H        + repmat(FDM_clim,[1,length(time)]) - FAC_GEMB;
    H_NF     = H_NF     + repmat(FDM_clim,[1,length(time)]); % add the climatological firn air content back in for this one
    H_static = H_static + repmat(FDM_clim,[1,length(time)]); % and this one

    % Account for thickness changes between the gate and the grounding line
    % (don't apply to 'static' or 'insitu' versions)
    gate_smb = double(gate_smb);
    H      = H      + repmat(gate_smb,[1,length(time)]);
    H_NF   = H_NF   + repmat(gate_smb,[1,length(time)]);
    H_NDH  = H_NDH  + repmat(gate_smb,[1,length(time)]);
    H_GSFC = H_GSFC + repmat(gate_smb,[1,length(time)]);
    H_GEMB = H_GEMB + repmat(gate_smb,[1,length(time)]);
    
    % After all those corrections, check that we haven't created any
    % unrealistic ice thickness
    H(H<0)               = 0;
    H_static(H_static<0) = 0;
    H_insitu(H_insitu<0) = 0;
    H_NDH(H_NDH<0)       = 0;
    H_NF(H_NF<0)         = 0;
    H_GSFC(H_GSFC<0)     = 0;
    H_GEMB(H_GEMB<0)     = 0;

    % Finally, make sure that we have NaNs in the right place
    for j = 1:length(time)
        H(gate_nan,j)        = NaN;
        H_static(gate_nan,j) = NaN;
        H_insitu(gate_nan,j) = NaN;
        H_NDH(gate_nan,j)    = NaN;
        H_NF(gate_nan,j)     = NaN;
        H_GSFC(gate_nan,j)   = NaN;
        H_GEMB(gate_nan,j)   = NaN;
    end

    %% Calculate discharge

    % Get flux gate sections
    sections = regionprops(~isnan(gate_x), gate_x,'pixelIdxList','PixelValues');

    for j = 1:numel(sections)

        % Get section data
        sect_lat = gate_lat(sections(j).PixelIdxList);
        sect_lon = gate_lon(sections(j).PixelIdxList);
        %   - merged bed, REMA surface, without adjustments
        sect_H = H(sections(j).PixelIdxList,:);
        sect_H_err = He(sections(j).PixelIdxList,:);
        sect_H_min = sect_H-sect_H_err;
        %sect_H_min(sect_H_min<0) = 0; % prepare this now so that we don't end up with any negative discharge
        %   - no thickness corrections applied at all
        sect_H_static = H_static(sections(j).PixelIdxList,:);
        sect_H_static_err = He_static(sections(j).PixelIdxList,:);
        sect_H_static_min = sect_H_static-sect_H_static_err;
        %sect_H_static_min(sect_H_static_min<0) = 0; % prepare this now so that we don't end up with any negative discharge
        %   - no downstream thickness correction applied
        sect_H_insitu = H_insitu(sections(j).PixelIdxList,:);
        sect_H_insitu_err = He_insitu(sections(j).PixelIdxList,:);
        sect_H_insitu_min = sect_H_insitu-sect_H_insitu_err;
        %sect_H_insitu_min(sect_H_insitu_min<0) = 0; % prepare this now so that we don't end up with any negative discharge
        %   - no dH applied
        sect_H_NDH = H_NDH(sections(j).PixelIdxList,:);
        sect_H_NDH_err = He_NDH(sections(j).PixelIdxList,:);
        sect_H_NDH_min = sect_H_NDH-sect_H_NDH_err;
        %sect_H_NDH_min(sect_H_NDH_min<0) = 0; % prepare this now so that we don't end up with any negative discharge
        %   - No firn correction applied
        sect_H_NF = H_NF(sections(j).PixelIdxList,:);
        sect_H_NF_err = He_NF(sections(j).PixelIdxList,:);
        sect_H_NF_min = sect_H_NF-sect_H_NF_err;
        %sect_H_NF_min(sect_H_NF_min<0) = 0; % prepare this now so that we don't end up with any negative discharge
        %   - GSFC firn correction applied
        sect_H_GSFC = H_GSFC(sections(j).PixelIdxList,:);
        sect_H_GSFC_err = He_GSFC(sections(j).PixelIdxList,:);
        sect_H_GSFC_min = sect_H_GSFC-sect_H_GSFC_err;
        %sect_H_GSFC_min(sect_H_GSFC_min<0) = 0; % prepare this now so that we don't end up with any negative discharge
        %   - GEMB firn correction applied
        sect_H_GEMB = H_GEMB(sections(j).PixelIdxList,:);
        sect_H_GEMB_err = He_GEMB(sections(j).PixelIdxList,:);
        sect_H_GEMB_min = sect_H_GEMB-sect_H_GEMB_err;
        %sect_H_GEMB_min(sect_H_GEMB_min<0) = 0; % prepare this now so that we don't end up with any negative discharge


        % Ice velocity
        sect_ctv = V.ctv(sections(j).PixelIdxList,:);
        sect_ctv_min = V.ctv_min(sections(j).PixelIdxList,:);
        sect_ctv_max = V.ctv_max(sections(j).PixelIdxList,:);

        %  Calculate distance between flux gate points
        d = pathdistps(sect_lat(:),sect_lon(:)); % Distance along flux gate
        dx = gradient(d); % Distance between each flux gate data point
        dx = repmat(dx(:),[1,length(time)]);

        % Calculate discharge
        %   - merged bed, REMA surface, without adjustments
        tmp                = sect_ctv.*sect_H.*dx.*rhoi.*1e-12;
        tmp_vel_max        = sect_ctv_max.*sect_H.*dx.*rhoi.*1e-12;
        tmp_vel_min        = sect_ctv_min.*sect_H.*dx.*rhoi.*1e-12;
        tmp_H_max          = sect_ctv.*(sect_H+sect_H_err).*dx.*rhoi.*1e-12;
        tmp_H_min          = sect_ctv.*sect_H_min.*dx.*rhoi.*1e-12;
        %   - no thickness corrections applied at all
        tmp_static         = sect_ctv.*sect_H_static.*dx.*rhoi.*1e-12;
        tmp_static_vel_max = sect_ctv_max.*sect_H_static.*dx.*rhoi.*1e-12;
        tmp_static_vel_min = sect_ctv_min.*sect_H_static.*dx.*rhoi.*1e-12;
        tmp_static_H_max   = sect_ctv.*(sect_H_static+sect_H_static_err).*dx.*rhoi.*1e-12;
        tmp_static_H_min   = sect_ctv.*sect_H_static_min.*dx.*rhoi.*1e-12;
        %   - no downstream correction applied
        tmp_insitu         = sect_ctv.*sect_H_insitu.*dx.*rhoi.*1e-12;
        tmp_insitu_vel_max = sect_ctv_max.*sect_H_insitu.*dx.*rhoi.*1e-12;
        tmp_insitu_vel_min = sect_ctv_min.*sect_H_insitu.*dx.*rhoi.*1e-12;
        tmp_insitu_H_max   = sect_ctv.*(sect_H_insitu+sect_H_insitu_err).*dx.*rhoi.*1e-12;
        tmp_insitu_H_min   = sect_ctv.*sect_H_insitu_min.*dx.*rhoi.*1e-12;
        %   - no dH applied 
        tmp_NDH          = sect_ctv.*sect_H_NDH.*dx.*rhoi.*1e-12;
        tmp_NDH_vel_max  = sect_ctv_max.*sect_H_NDH.*dx.*rhoi.*1e-12;
        tmp_NDH_vel_min  = sect_ctv_min.*sect_H_NDH.*dx.*rhoi.*1e-12;
        tmp_NDH_H_max    = sect_ctv.*(sect_H_NDH+sect_H_NDH_err).*dx.*rhoi.*1e-12;
        tmp_NDH_H_min    = sect_ctv.*sect_H_NDH_min.*dx.*rhoi.*1e-12;
        %   - no firn correction applied
        tmp_NF           = sect_ctv.*sect_H_NF.*dx.*rhoi.*1e-12;
        tmp_NF_vel_max   = sect_ctv_max.*sect_H_NF.*dx.*rhoi.*1e-12;
        tmp_NF_vel_min   = sect_ctv_min.*sect_H_NF.*dx.*rhoi.*1e-12;
        tmp_NF_H_max     = sect_ctv.*(sect_H_NF+sect_H_NF_err).*dx.*rhoi.*1e-12;
        tmp_NF_H_min     = sect_ctv.*sect_H_NF_min.*dx.*rhoi.*1e-12;
        %   - GSFC firn correction applied
        tmp_GSFC         = sect_ctv.*sect_H_GSFC.*dx.*rhoi.*1e-12;
        tmp_GSFC_vel_max = sect_ctv_max.*sect_H_GSFC.*dx.*rhoi.*1e-12;
        tmp_GSFC_vel_min = sect_ctv_min.*sect_H_GSFC.*dx.*rhoi.*1e-12;
        tmp_GSFC_H_max   = sect_ctv.*(sect_H_GSFC+sect_H_GSFC_err).*dx.*rhoi.*1e-12;
        tmp_GSFC_H_min   = sect_ctv.*sect_H_GSFC_min.*dx.*rhoi.*1e-12;
        %   - GEMB firn correction applied
        tmp_GEMB         = sect_ctv.*sect_H_GEMB.*dx.*rhoi.*1e-12;
        tmp_GEMB_vel_max = sect_ctv_max.*sect_H_GEMB.*dx.*rhoi.*1e-12;
        tmp_GEMB_vel_min = sect_ctv_min.*sect_H_GEMB.*dx.*rhoi.*1e-12;
        tmp_GEMB_H_max   = sect_ctv.*(sect_H_GEMB+sect_H_GEMB_err).*dx.*rhoi.*1e-12;
        tmp_GEMB_H_min   = sect_ctv.*sect_H_GEMB_min.*dx.*rhoi.*1e-12;

        % Concatenate
        if j == 1
            % merged bed, REMA surface, without adjustments
            Q         = [tmp;         nan(1,length(time))];
            Q_vel_max = [tmp_vel_max; nan(1,length(time))];
            Q_vel_min = [tmp_vel_min; nan(1,length(time))];
            Q_H_max   = [tmp_H_max;   nan(1,length(time))];
            Q_H_min   = [tmp_H_min;   nan(1,length(time))];
            % no thickness corrections
            Q_static         = [tmp_static;         nan(1,length(time))];
            Q_static_vel_max = [tmp_static_vel_max; nan(1,length(time))];
            Q_static_vel_min = [tmp_static_vel_min; nan(1,length(time))];
            Q_static_H_max   = [tmp_static_H_max;   nan(1,length(time))];
            Q_static_H_min   = [tmp_static_H_min;   nan(1,length(time))];
            % no downstream correction
            Q_insitu         = [tmp_insitu;         nan(1,length(time))];
            Q_insitu_vel_max = [tmp_insitu_vel_max; nan(1,length(time))];
            Q_insitu_vel_min = [tmp_insitu_vel_min; nan(1,length(time))];
            Q_insitu_H_max   = [tmp_insitu_H_max;   nan(1,length(time))];
            Q_insitu_H_min   = [tmp_insitu_H_min;   nan(1,length(time))];
            % no dH applied
            Q_NDH         = [tmp_NDH;         nan(1,length(time))];
            Q_NDH_vel_max = [tmp_NDH_vel_max; nan(1,length(time))];
            Q_NDH_vel_min = [tmp_NDH_vel_min; nan(1,length(time))];
            Q_NDH_H_max   = [tmp_NDH_H_max;   nan(1,length(time))];
            Q_NDH_H_min   = [tmp_NDH_H_min;   nan(1,length(time))];
            % no firn correction
            Q_NF         = [tmp_NF;         nan(1,length(time))];
            Q_NF_vel_max = [tmp_NF_vel_max; nan(1,length(time))];
            Q_NF_vel_min = [tmp_NF_vel_min; nan(1,length(time))];
            Q_NF_H_max   = [tmp_NF_H_max;   nan(1,length(time))];
            Q_NF_H_min   = [tmp_NF_H_min;   nan(1,length(time))];
            % GSFC firn correction
            Q_GSFC         = [tmp_GSFC;         nan(1,length(time))];
            Q_GSFC_vel_max = [tmp_GSFC_vel_max; nan(1,length(time))];
            Q_GSFC_vel_min = [tmp_GSFC_vel_min; nan(1,length(time))];
            Q_GSFC_H_max   = [tmp_GSFC_H_max;   nan(1,length(time))];
            Q_GSFC_H_min   = [tmp_GSFC_H_min;   nan(1,length(time))];
            % GEMB firn correction
            Q_GEMB         = [tmp_GEMB;         nan(1,length(time))];
            Q_GEMB_vel_max = [tmp_GEMB_vel_max; nan(1,length(time))];
            Q_GEMB_vel_min = [tmp_GEMB_vel_min; nan(1,length(time))];
            Q_GEMB_H_max   = [tmp_GEMB_H_max;   nan(1,length(time))];
            Q_GEMB_H_min   = [tmp_GEMB_H_min;   nan(1,length(time))];
        else
            % merged bed, REMA surface without adjustments
            Q         = [Q;tmp;                 nan(1,length(time))];
            Q_vel_max = [Q_vel_max;tmp_vel_max; nan(1,length(time))];
            Q_vel_min = [Q_vel_min;tmp_vel_min; nan(1,length(time))];
            Q_H_max   = [Q_H_max;tmp_H_max;     nan(1,length(time))];
            Q_H_min   = [Q_H_min;tmp_H_min;     nan(1,length(time))];
            % no thickness corrections applied
            Q_static         = [Q_static;tmp_static;                 nan(1,length(time))];
            Q_static_vel_max = [Q_static_vel_max;tmp_static_vel_max; nan(1,length(time))];
            Q_static_vel_min = [Q_static_vel_min;tmp_static_vel_min; nan(1,length(time))];
            Q_static_H_max   = [Q_static_H_max;tmp_static_H_max;     nan(1,length(time))];
            Q_static_H_min   = [Q_static_H_min;tmp_static_H_min;     nan(1,length(time))];
            % no downstream correction applied
            Q_insitu         = [Q_insitu;tmp_insitu;                 nan(1,length(time))];
            Q_insitu_vel_max = [Q_insitu_vel_max;tmp_insitu_vel_max; nan(1,length(time))];
            Q_insitu_vel_min = [Q_insitu_vel_min;tmp_insitu_vel_min; nan(1,length(time))];
            Q_insitu_H_max   = [Q_insitu_H_max;tmp_insitu_H_max;     nan(1,length(time))];
            Q_insitu_H_min   = [Q_insitu_H_min;tmp_insitu_H_min;     nan(1,length(time))];
            % no dH correction applied
            Q_NDH         = [Q_NDH;tmp_NDH;                 nan(1,length(time))];
            Q_NDH_vel_max = [Q_NDH_vel_max;tmp_NDH_vel_max; nan(1,length(time))];
            Q_NDH_vel_min = [Q_NDH_vel_min;tmp_NDH_vel_min; nan(1,length(time))];
            Q_NDH_H_max   = [Q_NDH_H_max;tmp_NDH_H_max;     nan(1,length(time))];
            Q_NDH_H_min   = [Q_NDH_H_min;tmp_NDH_H_min;     nan(1,length(time))];
            % no firn correction applied
            Q_NF         = [Q_NF;tmp_NF;                 nan(1,length(time))];
            Q_NF_vel_max = [Q_NF_vel_max;tmp_NF_vel_max; nan(1,length(time))];
            Q_NF_vel_min = [Q_NF_vel_min;tmp_NF_vel_min; nan(1,length(time))];
            Q_NF_H_max   = [Q_NF_H_max;tmp_NF_H_max;     nan(1,length(time))];
            Q_NF_H_min   = [Q_NF_H_min;tmp_NF_H_min;     nan(1,length(time))];
            % GSFC firn correction applied
            Q_GSFC         = [Q_GSFC;tmp_GSFC;                 nan(1,length(time))];
            Q_GSFC_vel_max = [Q_GSFC_vel_max;tmp_GSFC_vel_max; nan(1,length(time))];
            Q_GSFC_vel_min = [Q_GSFC_vel_min;tmp_GSFC_vel_min; nan(1,length(time))];
            Q_GSFC_H_max   = [Q_GSFC_H_max;tmp_GSFC_H_max;     nan(1,length(time))];
            Q_GSFC_H_min   = [Q_GSFC_H_min;tmp_GSFC_H_min;     nan(1,length(time))];
            % GEMB firn correction applied
            Q_GEMB         = [Q_GEMB;tmp_GEMB;                 nan(1,length(time))];
            Q_GEMB_vel_max = [Q_GEMB_vel_max;tmp_GEMB_vel_max; nan(1,length(time))];
            Q_GEMB_vel_min = [Q_GEMB_vel_min;tmp_GEMB_vel_min; nan(1,length(time))];
            Q_GEMB_H_max   = [Q_GEMB_H_max;tmp_GEMB_H_max;     nan(1,length(time))];
            Q_GEMB_H_min   = [Q_GEMB_H_min;tmp_GEMB_H_min;     nan(1,length(time))];
        end

    end
    clear tmp* sect*

    % Errors
    %   - all corrections
    Q_vel_err = (abs(Q_vel_max-Q) + abs(Q-Q_vel_min))./2;
    Q_H_err = (abs(Q_H_max-Q) + abs(Q-Q_H_min))./2;
    Q_err = sqrt(Q_vel_err.^2 + Q_H_err.^2); % assume velocity and thickness errors are independent
    %   - static (no thickness corrections applied)
    Q_static_vel_err = (abs(Q_static_vel_max-Q_static) + abs(Q_static-Q_static_vel_min))./2;
    Q_static_H_err = (abs(Q_static_H_max-Q_static) + abs(Q_static-Q_static_H_min))./2;
    Q_static_err = sqrt(Q_static_vel_err.^2 + Q_static_H_err.^2); % assume velocity and thickness errors are independent
    %   - insitu (no downstream correction)
    Q_insitu_vel_err = (abs(Q_insitu_vel_max-Q_insitu) + abs(Q_insitu-Q_insitu_vel_min))./2;
    Q_insitu_H_err = (abs(Q_insitu_H_max-Q_insitu) + abs(Q_insitu-Q_insitu_H_min))./2;
    Q_insitu_err = sqrt(Q_insitu_vel_err.^2 + Q_insitu_H_err.^2); % assume velocity and thickness errors are independent
    %   - NDH (no dH correction)
    Q_NDH_vel_err = (abs(Q_NDH_vel_max-Q_NDH) + abs(Q_NDH-Q_NDH_vel_min))./2;
    Q_NDH_H_err = (abs(Q_NDH_H_max-Q_NDH) + abs(Q_NDH-Q_NDH_H_min))./2;
    Q_NDH_err = sqrt(Q_NDH_vel_err.^2 + Q_NDH_H_err.^2); % assume velocity and thickness errors are independent
    %   - NF (no firn correction)
    Q_NF_vel_err = (abs(Q_NF_vel_max-Q_NF) + abs(Q_NF-Q_NF_vel_min))./2;
    Q_NF_H_err = (abs(Q_NF_H_max-Q_NF) + abs(Q_NF-Q_NF_H_min))./2;
    Q_NF_err = sqrt(Q_NF_vel_err.^2 + Q_NF_H_err.^2); % assume velocity and thickness errors are independent
    %   - GSFC firn correction, rather than FDM
    Q_GSFC_vel_err = (abs(Q_GSFC_vel_max-Q_GSFC) + abs(Q_GSFC-Q_GSFC_vel_min))./2;
    Q_GSFC_H_err = (abs(Q_GSFC_H_max-Q_GSFC) + abs(Q_GSFC-Q_GSFC_H_min))./2;
    Q_GSFC_err = sqrt(Q_GSFC_vel_err.^2 + Q_GSFC_H_err.^2); % assume velocity and thickness errors are independent
    %   - GEMB firn correction, rather than FDM
    Q_GEMB_vel_err = (abs(Q_GEMB_vel_max-Q_GEMB) + abs(Q_GEMB-Q_GEMB_vel_min))./2;
    Q_GEMB_H_err = (abs(Q_GEMB_H_max-Q_GEMB) + abs(Q_GEMB-Q_GEMB_H_min))./2;
    Q_GEMB_err = sqrt(Q_GEMB_vel_err.^2 + Q_GEMB_H_err.^2); % assume velocity and thickness errors are independent
    clear *vel_err *_H_err
    
    % Make single precision
    Q = single(Q);
    Q_err = single(Q_err);
    Q_insitu = single(Q_insitu);
    Q_insitu_err = single(Q_insitu_err);
    Q_static = single(Q_static);
    Q_static_err = single(Q_static_err);
    Q_NF = single(Q_NF);
    Q_NF_err = single(Q_NF_err);
    Q_NDH = single(Q_NDH);
    Q_NDH_err = single(Q_NDH_err);
    Q_GSFC = single(Q_GSFC);
    Q_GSFC_err = single(Q_GSFC_err);
    Q_GEMB = single(Q_GEMB);
    Q_GEMB_err = single(Q_GEMB_err);

    
    %% Save pixel-based discharge time-series

    filled = V.filled;
    velocity_source = V.source_vel;
    save(['../out/int/discharge/pixel/discharge_timeseries_migrate_' num2str(migration_distance) 'm_' H_datasets{i} '_ns.mat'],...
        'Q','Q_err',...
        'Q_insitu','Q_insitu_err',...
        'Q_static','Q_static_err',...
        'Q_NF','Q_NF_err',...
        'Q_NDH','Q_NDH_err',...
        'Q_GSFC','Q_GSFC_err',...
        'Q_GEMB','Q_GEMB_err',...
        'time','filled','velocity_source');
		

    %% Calculate timeseries for each basin

    % basin info
    load('../out/int/basins/all_basins.mat');

    % Pre-allocate
    %   - discharge
    bQ            = zeros([numel(basins.name),length(time)]).*NaN;
    bQ_insitu     = zeros([numel(basins.name),length(time)]).*NaN;
    bQ_static     = zeros([numel(basins.name),length(time)]).*NaN;
    bQ_NF         = zeros([numel(basins.name),length(time)]).*NaN;
    bQ_NDH        = zeros([numel(basins.name),length(time)]).*NaN;
    bQ_GSFC       = zeros([numel(basins.name),length(time)]).*NaN;
    bQ_GEMB       = zeros([numel(basins.name),length(time)]).*NaN;
    bQE           = zeros([numel(basins.name),length(time)]).*NaN;
    bQE_insitu    = zeros([numel(basins.name),length(time)]).*NaN;
    bQE_static    = zeros([numel(basins.name),length(time)]).*NaN;
    bQE_NF        = zeros([numel(basins.name),length(time)]).*NaN;
    bQE_NDH       = zeros([numel(basins.name),length(time)]).*NaN;
    bQE_GSFC      = zeros([numel(basins.name),length(time)]).*NaN;
    bQE_GEMB      = zeros([numel(basins.name),length(time)]).*NaN;
    bQEsum        = zeros([numel(basins.name),length(time)]).*NaN;
    bQEsum_insitu = zeros([numel(basins.name),length(time)]).*NaN;
    bQEsum_static = zeros([numel(basins.name),length(time)]).*NaN;
    bQEsum_NF     = zeros([numel(basins.name),length(time)]).*NaN;
    bQEsum_NDH    = zeros([numel(basins.name),length(time)]).*NaN;
    bQEsum_GSFC   = zeros([numel(basins.name),length(time)]).*NaN;
    bQEsum_GEMB   = zeros([numel(basins.name),length(time)]).*NaN;
    %   - proportion of discharge observed
    bQp        = zeros([numel(basins.name),length(time)]).*NaN;
    bQp_insitu = zeros([numel(basins.name),length(time)]).*NaN;
    bQp_static = zeros([numel(basins.name),length(time)]).*NaN;
    bQp_NF     = zeros([numel(basins.name),length(time)]).*NaN;
    bQp_NDH    = zeros([numel(basins.name),length(time)]).*NaN;
    bQp_GSFC   = zeros([numel(basins.name),length(time)]).*NaN;
    bQp_GEMB   = zeros([numel(basins.name),length(time)]).*NaN;

    % Make copies where filled discharge is set to nan
    filled = logical(filled);
    Qf = Q;
    Qf(filled) = 0;
    Qf_insitu = Q_insitu;
    Qf_insitu(filled) = 0;
    Qf_static = Q_static;
    Qf_static(filled) = 0;
    Qf_NF = Q_NF;
    Qf_NF(filled) = 0;
    Qf_NDH = Q_NDH;
    Qf_NDH(filled) = 0;
    Qf_GSFC = Q_GSFC;
    Qf_GSFC(filled) = 0;
    Qf_GEMB = Q_GEMB;
    Qf_GEMB(filled) = 0;

    % calculate discharge
    for j = 1:numel(basins.name)
        if j == 1
            tf = logical(ones([length(gate_x),1]));
            % there is a nan splitting the main ice sheet from wilkins
            % island. To root-sum-square the errors for the AIS-wide
            % estimate, we need to set that nan to 0...
            isn = isnan(Q_err);
            Q_err(isn)=0;
            Q_insitu_err(isn) = 0;
            Q_static_err(isn) = 0;
            Q_NF_err(isn) = 0;
            Q_NDH_err(isn) = 0;
            Q_GSFC_err(isn) = 0;
            Q_GEMB_err(isn) = 0;
        else
            tf = inpoly2([gate_x(:),gate_y(:)],[basins.x{j},basins.y{j}]);
            
            % Sometimes we end up with various little segments that need to
            % be joined
            sections = regionprops(tf==1, tf,'pixelIdxList','PixelValues');
            if numel(sections)>1
                tf = double(tf);
                tf(tf==0) = NaN;
                tf = fillmissing(double(tf),'linear','EndValues','none','MaxGap',500); % limit the infill size to try and stop it wrapping round...
                tf(isnan(tf)) = 0;
                tf = logical(tf);
            end
        end
        
        % Some of the peninsula basins don't intersect any gate. Some
        % upstream migrated gates end up coming out of the basin, we can
        % have empty tf sometimes. Keep those as nan?
        if ~isempty(tf)
            % Discharge
            %   - main
            bQ(j,:)  = nansum(Q(tf,:),1);
            bQE(j,:) = rssq(Q_err(tf,:),1);
            Qmax = nansum(Q(tf,:)+Q_err(tf,:));
            Qmin = nansum(Q(tf,:)-Q_err(tf,:));
            bQEsum(j,:) = abs(abs(Qmax-bQ(j,:)) + abs(bQ(j,:)-Qmin))./2;
            %   - corrections
            % -- insitu
            bQ_insitu(j,:)  = nansum(Q_insitu(tf,:),1);
            bQE_insitu(j,:) = rssq(Q_insitu_err(tf,:),1);
            Qmax = nansum(Q_insitu(tf,:)+Q_insitu_err(tf,:));
            Qmin = nansum(Q_insitu(tf,:)-Q_insitu_err(tf,:));
            bQEsum_insitu(j,:) = abs(abs(Qmax-bQ_insitu(j,:)) + abs(bQ_insitu(j,:)-Qmin))./2;
            % -- static
            bQ_static(j,:)  = nansum(Q_static(tf,:),1);
            bQE_static(j,:) = rssq(Q_static_err(tf,:),1);
            Qmax = nansum(Q_static(tf,:)+Q_static_err(tf,:));
            Qmin = nansum(Q_static(tf,:)-Q_static_err(tf,:));
            bQEsum_static(j,:) = abs(abs(Qmax-bQ_static(j,:)) + abs(bQ_static(j,:)-Qmin))./2;
            % -- no firn
            bQ_NF(j,:)  = nansum(Q_NF(tf,:),1);
            bQE_NF(j,:) = rssq(Q_NF_err(tf,:),1);
            Qmax = nansum(Q_NF(tf,:)+Q_NF_err(tf,:));
            Qmin = nansum(Q_NF(tf,:)-Q_NF_err(tf,:));
            bQEsum_NF(j,:) = abs(abs(Qmax-bQ_NF(j,:)) + abs(bQ_NF(j,:)-Qmin))./2;
            % -- no dH
            bQ_NDH(j,:)  = nansum(Q_NDH(tf,:),1);
            bQE_NDH(j,:) = rssq(Q_NDH_err(tf,:),1);
            Qmax = nansum(Q_NDH(tf,:)+Q_NDH_err(tf,:));
            Qmin = nansum(Q_NDH(tf,:)-Q_NDH_err(tf,:));
            bQEsum_NDH(j,:) = abs(abs(Qmax-bQ_NDH(j,:)) + abs(bQ_NDH(j,:)-Qmin))./2;
            %   - alternative firn models
            % -- GSFC
            bQ_GSFC(j,:)  = nansum(Q_GSFC(tf,:),1);
            bQE_GSFC(j,:) = rssq(Q_GSFC_err(tf,:),1);
            Qmax = nansum(Q_GSFC(tf,:)+Q_GSFC_err(tf,:));
            Qmin = nansum(Q_GSFC(tf,:)-Q_GSFC_err(tf,:));
            bQEsum_GSFC(j,:) = abs(abs(Qmax-bQ_GSFC(j,:)) + abs(bQ_GSFC(j,:)-Qmin))./2;
            % -- GEMB
            bQ_GEMB(j,:)  = nansum(Q_GEMB(tf,:),1);
            bQE_GEMB(j,:) = rssq(Q_GEMB_err(tf,:),1);
            Qmax = nansum(Q_GEMB(tf,:)+Q_GEMB_err(tf,:));
            Qmin = nansum(Q_GEMB(tf,:)-Q_GEMB_err(tf,:));
            bQEsum_GEMB(j,:) = abs(abs(Qmax-bQ_GEMB(j,:)) + abs(bQ_GEMB(j,:)-Qmin))./2;

            % Proportion of discharge observed
            bQp(j,:) = nansum(Qf(tf,:),1)./bQ(j,:).*100;
            bQp_insitu(j,:) = nansum(Qf_insitu(tf,:),1)./bQ_insitu(j,:).*100;
            bQp_static(j,:) = nansum(Qf_static(tf,:),1)./bQ_static(j,:).*100;
            bQp_NF(j,:) = nansum(Qf_NF(tf,:),1)./bQ_NF(j,:).*100;
            bQp_NDH(j,:) = nansum(Qf_NDH(tf,:),1)./bQ_NDH(j,:).*100;
            %   - alternative firn models
            bQp_GSFC(j,:) = nansum(Qf_GSFC(tf,:),1)./bQ_GSFC(j,:).*100;
            bQp_GEMB(j,:) = nansum(Qf_GEMB(tf,:),1)./bQ_GEMB(j,:).*100;
        end
        
        if j == 1
            Q_err(isn) = NaN;
            Q_insitu_err(isn) = NaN;
            Q_static_err(isn) = NaN;
            Q_NF_err(isn) = NaN;
            Q_NDH_err(isn) = NaN;
            Q_GSFC_err(isn) = NaN;
            Q_GEMB_err(isn) = NaN;
        end
        
    end
    
    % Smooth the basin-scale time-series a bit
    %   - prep time
    a = -0.000001;
    b = 0.000001;
    r = a + (b-a).*rand(1,length(time));
    tmp = time + r;
    tmp = sort(tmp);
    %   - smooth with a moving mean filter just larger than one year, but
    %   only apply prior to July 2015
    target = time<=datenum(2015,7,1);
    bQ(:,target) = movmean(bQ(:,target),30*13,2,'omitnan','samplepoints',tmp(target));
    bQ_insitu(:,target) = movmean(bQ_insitu(:,target),30*13,2,'omitnan','samplepoints',tmp(target));
    bQ_static(:,target) = movmean(bQ_static(:,target),30*13,2,'omitnan','samplepoints',tmp(target));
    bQ_NF(:,target) = movmean(bQ_NF(:,target),30*13,2,'omitnan','samplepoints',tmp(target));
    bQ_NDH(:,target) = movmean(bQ_NDH(:,target),30*13,2,'omitnan','samplepoints',tmp(target));
    bQ_GSFC(:,target) = movmean(bQ_GSFC(:,target),30*13,2,'omitnan','samplepoints',tmp(target));
    bQ_GEMB(:,target) = movmean(bQ_GEMB(:,target),30*13,2,'omitnan','samplepoints',tmp(target));

    % make single precision
    bQ = single(bQ); bQE = single(bQE); bQp = single(bQp); bQEsum = single(bQEsum);
    bQ_insitu = single(bQ_insitu); bQE_insitu = single(bQE_insitu); bQp_insitu = single(bQp_insitu); bQEsum_insitu = single(bQEsum_insitu);
    bQ_static = single(bQ_static); bQE_static = single(bQE_static); bQp_static = single(bQp_static); bQEsum_static = single(bQEsum_static);
    bQ_NF = single(bQ_NF); bQE_NF = single(bQE_NF); bQp_NF = single(bQp_NF); bQEsum_NF = single(bQEsum_NF);
    bQ_NDH = single(bQ_NDH); bQE_NDH = single(bQE_NDH); bQp_NDH = single(bQp_NDH); bQEsum_NDH = single(bQEsum_NDH);
    bQ_GSFC = single(bQ_GSFC); bQE_GSFC = single(bQE_GSFC); bQp_GSFC = single(bQp_GSFC); bQEsum_GSFC = single(bQEsum_GSFC);
    bQ_GEMB = single(bQ_GEMB); bQE_GEMB = single(bQE_GEMB); bQp_GEMB = single(bQp_GEMB); bQEsum_GEMB = single(bQEsum_GEMB);

    % Save as .mat file
    save(['../out/int/discharge/basin/basin_discharge_migrate_' num2str(migration_distance) 'm_' H_datasets{i} '_all_basins_ns.mat'],...
        'bQ','bQE','bQp','bQEsum',...
        'bQ_insitu','bQE_insitu','bQp_insitu','bQEsum_insitu',...
        'bQ_static','bQE_static','bQp_static','bQEsum_static',...
        'bQ_NF','bQE_NF','bQp_NF','bQEsum_NF',...
        'bQ_NDH','bQE_NDH','bQp_NDH','bQEsum_NDH',...
        'bQ_GSFC','bQE_GSFC','bQp_GSFC','bQEsum_GSFC',...
        'bQ_GEMB','bQE_GEMB','bQp_GEMB','bQEsum_GEMB',...
        'time','velocity_source');
    
    %% Adjust bed elevation to match observed mass change
    
    % Calculate elevation adjustment using FrankenBed discharge
    if strcmp(H_datasets{i},'FrankenBed')
        
        % Pre-allocate output (set them to zero, so if this fails we just
        % won't modify the thickness at all)
        H_change = zeros(size(gate_x));
        H_new = zeros(size(gate_x));
        basin_H_change = zeros([numel(basins.name),1]);
        basin_H_inverted = zeros([numel(basins.name),1]);
        basin_H_orig = zeros([numel(basins.name),1]);
        
        % load inverted discharge
        inv = load('../out/int/discharge/inverted/inverted_discharge.mat'); % this is the discharge we need to match the timeseries of dH/dt

        % Use average observed discharge during overlap with SMB and dHdt
        % but also avoid the period where we see shifts in discharge due to
        % resolution
        keep = time>=datenum(1996,1,1) & time<=datenum(2014,1,1);
        
        % Get the average velocity, thickness and discharge during the
        % period of overlap
        H0(H0<0) = 1e-3; % set very small thickness in any places with impossible thickness
        V_avg = nanmean(V.ctv(:,keep),2);
        H_avg = nanmean(H0(:,keep),2);
        H_avg(H_avg<0) = 1e-3; % somehow, some negative thickness still persists occasionally, so do this again
        
        %  Calculate distance between flux gate points
        sections = regionprops(~isnan(gate_x), gate_x,'pixelIdxList','PixelValues');
        dx = zeros(size(gate_x)).*NaN;
        for j = 1:numel(sections)
            sect_lat = gate_lat(sections(j).PixelIdxList);
            sect_lon = gate_lon(sections(j).PixelIdxList);
            d = pathdistps(sect_lat(:),sect_lon(:)); % Distance along flux gate
            dx(sections(j).PixelIdxList) = gradient(d); % Distance between each flux gate data point
        end
        
        % Keep a copy of the original thickness
        H_avg0 = H_avg;
        
        % Calculate thickness scaling for MEaSUREs glacier basins
        basin_idx = find(basins.type_ID==2);
        for j = 1:numel(basin_idx)
            
            % Only bother if the discharge inversion worked
            if inv.Qinv(basin_idx(j))>0 && isfinite(inv.Qinv(basin_idx(j))) && ~isinf(inv.Qinv(basin_idx(j)))

                % Get location of pixels in this basin
                tf = inpoly2([gate_x(:),gate_y(:)],[basins.x{basin_idx(j)},basins.y{basin_idx(j)}]);
                sections = regionprops(tf==1, tf,'pixelIdxList','PixelValues');
                if numel(sections)>1
                    tf = double(tf);
                    tf(tf==0) = NaN;
                    tf = fillmissing(double(tf),'linear','EndValues','none','MaxGap',500); % limit the infill size to try and stop it wrapping round...
                    tf(isnan(tf)) = 0;
                    tf = logical(tf);
                end
                
                if ~isempty(tf)

                    % Calculate the discharge in this basin (time average
                    % during the above time period)
                    pixel_Q = V_avg(tf).*H_avg(tf).*dx(tf).*rhoi.*1e-12;
                    basin_Q  = nansum(pixel_Q);
                    resid_Q = basin_Q - inv.Qinv(basin_idx(j));
                    percResid_Q = resid_Q / inv.Qinv(basin_idx(j)) * 100;
                    percResid_Q0 = percResid_Q; % keep a copy of the original

                    % Iteratively adjust the thickness in these pixels until we
                    % hit the right discharge (or within 0.1% of it)
                    numIterations = 0;
                    while abs(percResid_Q)>0.1 && numIterations<1000 
                        % Initially assume that we have too little discharge
                        % and we need to get some more by increasing ice
                        % thickness
                        fac = 1;
                        if resid_Q > 0 % but if we have too much discharge, set a factor to make things the right sign
                            fac = -1;
                        end
                        % Prevent numerical errors
                        if percResid_Q == 0 || isinf(percResid_Q)
                            percResid_Q = 1e-5;
                        end
                        % Modify the thickness
                        H_avg(tf) = H_avg(tf) + (H_avg(tf).*(abs(percResid_Q)/100*fac));
                        % Recalculate discharge
                        pixel_Q = V_avg(tf).*H_avg(tf).*dx(tf).*rhoi.*1e-12;
                        basin_Q  = nansum(pixel_Q);
                        resid_Q = basin_Q - inv.Qinv(basin_idx(j));
                        percResid_Q = resid_Q / inv.Qinv(basin_idx(j)) * 100;
                        % Prevent numerical errors
                        if percResid_Q == 0 || isinf(percResid_Q)
                            percResid_Q = 1e-5;
                        end
                        % increment counter
                        numIterations = numIterations+1;
                    end
                    if numIterations==1000
                        disp(' ');
                        disp(['Warning maximum number of iterations reached for basin ' basins.name{basin_idx(j)}]);
                        disp(['Final percentage residual was: ' num2str(percResid_Q) ' % (compared to the initial: ' num2str(percResid_Q0) '%)']);
                    else
                        disp(['Final percentage residual for ' basins.name{basin_idx(j)} ' was: ' num2str(percResid_Q) ' % (compared to the initial: ' num2str(percResid_Q0) '%). NumIters = ' num2str(numIterations) ]);
                    end

                    % Keep track of new thicknesses
                    H_new(tf) = H_avg(tf);
                    H_change(tf) = H_avg(tf) - H_avg0(tf); % positive means we have increased ice thickness to achive the necessary discharge
                    basin_H_change(basin_idx(j)) = nanmean(H_change(tf));  
                    basin_H_inverted(basin_idx(j)) = nanmean(H_avg(tf));
                    basin_H_orig(basin_idx(j)) = nanmean(H_avg0(tf));
                    
                end
                
            end
            
        end
        
        % Clear some space
        clear inv keep V_avg H_avg H_avg0 basin_idx j inv tf sections pixel_Q basin_Q resid_Q percResid_Q numIterations

        
        % Make sure we haven't created any weird thickness or produce
        % unrealistically large changes in thickness
        remove = isinf(H_change) | isnan(H_change) | H_new<=0;
        H_change(remove) = 0; 
            
        % Keep a record of the thickness multiplier
        save(['../out/int/derived/thickness_adjustment_' num2str(migration_distance) 'm_ns.mat'],'H_change','H_new','basin_H_change','basin_H_inverted','basin_H_orig');
        
    end 

    
end

end




